local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Log = BaikCore.Log
local Option = BaikCore.Option

-- Add Module
local Border = BaikUnitFrame:NewModule("Border", "AceHook-3.0")

-- Constants
local function CreateColor(r, g, b, a)
    Assert:Number(r)
    Assert:Number(g)
    Assert:Number(b)
    Assert:Number(a)

    local color = {}
    color.r = r
    color.g = g
    color.b = b
    color.a = a

    return color
end

local BORDER_COLOR_DB = "unit_border_color"
local COLOR_DEFAULT = CreateColor(1.0, 1.0, 1.0, 1.0)
local BORDERS = {
    PlayerFrameTexture,
    TargetFrameTextureFrameTexture,
    FocusFrameTextureFrameTexture,
    PartyMemberFrame1Texture,
    PartyMemberFrame2Texture,
    PartyMemberFrame3Texture,
    PartyMemberFrame4Texture,
    Boss1TargetFrameTextureFrameTexture,
    Boss2TargetFrameTextureFrameTexture,
    Boss3TargetFrameTextureFrameTexture,
    Boss4TargetFrameTextureFrameTexture,
    Boss5TargetFrameTextureFrameTexture,
}

local function SyncBorders(r, g, b, a)
    Assert:Number(r)
    Assert:Number(g)
    Assert:Number(b)
    Assert:Number(a)
    for _, border in pairs(BORDERS) do
        if border then
            border:SetVertexColor(r, g, b, a)
        end
    end
end

local function GetOpt()
    return {
        name = "Border",
        type = "group",
        args = {
            color = {
                name = "Color",
                type = "color",
                hasAlpha = true,
                get = function(...)
                    local color = Database:Get(BORDER_COLOR_DB)
                    return color.r, color.g, color.b, color.a
                end,
                set = function(self, r, g, b, a, ...)
                    Database:Set(BORDER_COLOR_DB, CreateColor(r, g, b, a))
                    SyncBorders(r, g, b, a)
                end,
            }
        }
    }
end

-- Ace3 Callbacks
function Border:OnInitialize()
    Log:v("Border_OnInitialize")
    Database:InitIfEmpty(BORDER_COLOR_DB, COLOR_DEFAULT)
    Option:AddOption(BaikUnitFrame.OPTION_NAME, "border", GetOpt())
end

function Border:OnEnable()
    Log:v("Border_OnEnable")
    local color = Database:Get(BORDER_COLOR_DB)
    SyncBorders(color.r, color.g, color.b, color.a)
end

function Border:OnDisable()
    Log:v("Border_OnDisable")
end
